/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.junit.prueffall1;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.rdstmc.junit.RDSTMCPrueffall;
import de.bsvrz.ste.rdstmc.junit.tools.WarteAufAktualisierteDaten;
import de.bsvrz.ste.rdstmc.meldungserstellung.VerwaltungMeldungen;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * @author Sans
 *
 */
public class Prueffall1_2_300s extends RDSTMCPrueffall
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  @BeforeClass
  public static void setUpBeforeClass() throws Exception 
  {
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
  }

  @Before
  public void setUp() throws Exception 
  {
  }

  @After
  public void tearDown() throws Exception
  {
  }
    
  @Test
  public void testPrueffall_1_2_300s()throws Exception
  {
    assertTestGeneratorLaeuft();

    SystemObject meldung2 = getDataModel().getObject(VerwaltungMeldungen.PID_PREFIX + "2");
    assertNotNull("Objekt " + VerwaltungMeldungen.PID_PREFIX + "2" + " existiert", meldung2);
    
    SystemObject meldung3 = getDataModel().getObject(VerwaltungMeldungen.PID_PREFIX + "3");
    assertNotNull("Objekt " + VerwaltungMeldungen.PID_PREFIX + "3" + " existiert", meldung3);
    
    WarteAufAktualisierteDaten waad = new WarteAufAktualisierteDaten(getConnection(), getMeldungsMenge(), 
        new SystemObject[] { meldung2, meldung3 }, getDataDescription(), 40000 + TOL_PLUS);
    
    ResultData rd2 = waad.getAktualisierteDaten().get(meldung2.getPid());
    ResultData rd3 = waad.getAktualisierteDaten().get(meldung3.getPid());
    
    assertTrue("2 Meldungen " + meldung2.getPid() + ", " + meldung3.getPid() + " wurden aktualisiert",
        waad.getAktualisierteDaten().size() == 2 &&
        rd2 != null && rd2.getObject().equals(meldung2) && 
        rd3 != null && rd3.getObject().equals(meldung3));

    assertPruefZeitKorrekt(300000, Math.max(rd2.getDataTime(), rd3.getDataTime()));
            
    assertMeldungsDatenKorrekt(rd2,
        InformationsTyp.ALERTC,
        MeldungsArt.AKTUALISIERUNG,
        GeografischeBedeutung.NATIONAL,
        ZeitlicheDringlichkeit.NORMAL,
        ZeitlicheDauer.UNBEKANNT,
        -1,
        103,
        11907,
        null,
        null);
    
    assertMeldungsDatenKorrekt(rd3,
        InformationsTyp.ALERTC,
        MeldungsArt.AKTUALISIERUNG,
        GeografischeBedeutung.NATIONAL,
        ZeitlicheDringlichkeit.NORMAL,
        ZeitlicheDauer.UNBEKANNT,
        -1,
        108,
        11906,
        null,
        null);
  }

}
