/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.junit.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;

/**
 * Erstellt eine DAV-Verbindung vom Typ <code>ClientDavInterface</code> aus den Daten einer 
 * Property-Datei
 * 
 * @author Sans
 *
 */
public class DavConnectionFromProperties 
{
  static final String PROPERTY_FILE_NAME = "DavConnection.properties";
  
  static final String DAV_KEY = "datenverteiler";
  static final String USER_KEY = "benutzer";
  static final String AUTH_KEY = "authentifizierung";
  
  public DavConnectionFromProperties()
  {
    Properties properties = new Properties();
    File propertyFile = new File(PROPERTY_FILE_NAME);
    try
    {
      properties.load(new FileInputStream(propertyFile));
    }
    catch (FileNotFoundException e)
    {
      // throw new RuntimeException("Datei nicht gefunden: " + PROPERTY_FILE_NAME, e);
      
      // Falls Datei nicht existiert sind properties leer und es werden Standardwerte genommen
    }
    catch (IOException e)
    {
      throw new RuntimeException("Fehler beim Zugriff auf Datei " + PROPERTY_FILE_NAME, e);
    }
    
    String datenverteiler = properties.getProperty(DAV_KEY);
    if (datenverteiler == null || datenverteiler.equals(""))
    {
      // throw new RuntimeException("Property " + DAV_KEY + " ist nicht angegeben");
      
      // Standardwert fr Datenverteiler
      datenverteiler = "127.0.0.1:8083";
    }
    
    String benutzer = properties.getProperty(USER_KEY);
    if (benutzer == null || benutzer.equals(""))
    {
      // throw new RuntimeException("Property " + USER_KEY + " ist nicht angegeben");
      
      // Standardwert fr Benutzer
      benutzer = "Tester";
    }
    
    String authentifizierung = properties.getProperty(AUTH_KEY);
    if (authentifizierung == null || authentifizierung.equals(""))
    {
      // throw new RuntimeException("Property " + AUTH_KEY + " ist nicht angegeben");
      
      // Standardwert fr Authentifizierung
      authentifizierung = "passwd";
    }
    
    String[] davParameter = new String[] { "-datenverteiler=" + datenverteiler,
                             "-benutzer=" + benutzer,
                             "-authentifizierung=" + authentifizierung };
    try
    {
      _connection = new ClientDavConnection(new ClientDavParameters(davParameter));
      _connection.connect();
      _connection.login();
    }
    catch (Exception e)
    {
      e.printStackTrace();
      throw new RuntimeException("Fehler beim Aufbauen der Datenverteiler-Verbindung", e);
    }
  }
  
  private ClientDavInterface _connection = null;
  
  public ClientDavInterface getConnection()
  {
    return _connection;
  }
  
}
