/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.konfiguration;

import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein StrassenAbschnitt modelliert.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:49:25 $ / ($Author: Sans $)
 */
public class StrassenAbschnitt extends DavProxyObjekt 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Messquerschnitt, an dem dieser Strassenabschnitt anfngt
   */
  private MessQuerschnitt _vonMq;
    
  /**
   * Messquerschnitt, an dem dieser Strassenabschnitt endet
   */
  private MessQuerschnitt _nachMq;
  
  /**
   * Erstellt eine neue Instanz vom Typ <code>StrassenAbschnitt</code>
   * 
   * @param davObjekt das zuzuordnente Dav-Objekt
   */
  public StrassenAbschnitt(SystemObject davObjekt, MessQuerschnitt vonMq, MessQuerschnitt nachMq)
  {
    super(davObjekt);
    _vonMq = vonMq;
    _nachMq = nachMq;
  }
  
  /**
   * @return den Messquerschnitt, an dem dieser Strassenabschnitt anfngt
   */
  public MessQuerschnitt getVonMq()
  {
    return _vonMq;
  }

  /**
   * @return den Messquerschnitt, an dem dieser Strassenabschnitt endet
   */
  public MessQuerschnitt getNachMq()
  {
    return _nachMq;
  }
  
}
