/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.konfiguration;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Es wird ein TMC-Location-Code (Tmc-Punkt, Tmc-Gebiet) modelliert.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.2 $ / $Date: 2010/03/09 13:46:17 $ / ($Author: Sans $)
 */
public class TmcLocationPunkt extends DavProxyObjekt 
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
  
  /**
   * Erstellt eine neue Instanz vom Typ <code>TmcLocationPunkt</code>
   * 
   * @param davObjekt das zuzuordnente Dav-Objekt
   * @param laenge die Lnge in m des Strassensegments
   */
  public TmcLocationPunkt(SystemObject davObjekt, DataModel dataModel) 
  {
    super(davObjekt);
    
    if (!davObjekt.isOfType("typ.tmcPunkt"))
    {
      throw new IllegalArgumentException("bergebenes Objekt ist nicht vom Typ typ.tmcPunkt");
    }
    
    AttributeGroup atg = dataModel.getAttributeGroup("atg.tmcLocationCode");
    Data data = davObjekt.getConfigurationData(atg);
    _locationCode = data.getItem("TmcLocationCode").asUnscaledValue().intValue();
    _strassenNummer = data.getItem("TmcStraenNummer").asTextValue().getText();
    _ausfahrtNummer = data.getItem("AusfahrtNr").asTextValue().getText();
    _ortsTyp = data.getItem("TmcOrtsTyp").asUnscaledValue().intValue();
    
    _x = data.getItem("x").asScaledValue().doubleValue();
    _y = data.getItem("y").asScaledValue().doubleValue();
    
    atg = dataModel.getAttributeGroup("atg.tmcPunkt");
    data = davObjekt.getConfigurationData(atg);
   
    SystemObject obj = data.getItem("istTeilvonTmcLinie").asReferenceValue().getSystemObject();
    if (obj != null)
    {
      _linienCode = holeTmcCode(obj, dataModel);
    }
    obj = data.getItem("istTeilvonTmcGebiet").asReferenceValue().getSystemObject();
    if (obj != null)
    {
      _gebietsCode = holeTmcCode(obj, dataModel);
    }
  }
  
  private int holeTmcCode(SystemObject obj, DataModel dataModel)
  {
    AttributeGroup atg = dataModel.getAttributeGroup("atg.tmcLocationCode");
    Data data = obj.getConfigurationData(atg);
    return data.getItem("TmcLocationCode").asUnscaledValue().intValue();
  }
  
  // TmcLocationCode
  private int _locationCode;
  private String _strassenNummer;
  private String _ausfahrtNummer;
  private int _ortsTyp;
  
  // Koordinaten
  private double _x;
  private double _y;
  
  // Tmc-Punkt
  private TmcLocationPunkt _vorgaenger;
  private TmcLocationPunkt _nachfolger;
  private int _linienCode;
  private int _gebietsCode;
  
  public int getLocationCode()
  {
    return _locationCode;
  }
  
  public String getStrassenNummer()
  {
    return _strassenNummer;
  }
  
  public String getAusfahrtNummer()
  {
    return _ausfahrtNummer;
  }
  
  public int getOrtsTyp()
  {
    return _ortsTyp;
  }
  
  public double getX()
  {
    return _x;
  }
  
  public double getY()
  {
    return _y;
  }
  
  public int getVorgaengerCode()
  {
    return _vorgaenger != null ? _vorgaenger.getLocationCode() : 0;
  }
  
  public int getNachfolgerCode()
  {
    return _nachfolger != null ? _nachfolger.getLocationCode() : 0;
  }
  
  public int getLinienCode()
  {
    return _linienCode;
  }
  
  public int getGebietsCode()
  {
    return _gebietsCode;
  }
  
  public TmcLocationPunkt getVorgaenger()
  {
    return _vorgaenger;
  }
  
  void setVorgaenger(TmcLocationPunkt vorgaenger)
  {
    _vorgaenger = vorgaenger;
  }
  
  public TmcLocationPunkt getNachfolger()
  {
    return _nachfolger;
  }
  
  void setNachfolger(TmcLocationPunkt nachfolger)
  {
    _nachfolger = nachfolger;
  }
  
}
