/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.test;

import java.util.List;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.rdstmc.allgemein.StoerfallZustand;
import de.bsvrz.sys.funclib.debug.Debug;

public class Prueffall1 extends PrueffallBasis implements Runnable
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();

  private final static String STAU_MENGE = "Staus";
  private final static String STAU_PID_PREFIX = "situation.StauRDSTMC";
  private final static String STAU_NAME_PREFIX = "StauRDSTMC";
  
  private boolean _appStarten = false;
  
  public Prueffall1(ClientDavInterface connection, boolean appStarten)
  {
    super(connection);
    
    _appStarten = appStarten;
    bereinigeRdsMeldungen();
    Runtime.getRuntime().addShutdownHook(new ShutdownHook());
  }
  
  public Prueffall1(ClientDavInterface connection)
  {
    this(connection, false);
  }
    
  public class ShutdownHook extends Thread
  {
    public void run()
    {
      if (getApplikationsProzess() != null)
      {
        getApplikationsProzess().destroy();
        System.out.println("Zu prfende Applikation gestoppt");    
      }
      parameterRestaurieren();
      bereinigeStaus();
    }
  }
    
  public void run()
  {
    _debug.info("Beginn der Prfungen Prffall 1\nStarte P.1.1");
    setzteIndikatoren(new int[] {2, 2, 2, 3, 2, 3, 3, 3, 3, 3, 3, 4, 5, 1, 1, 3, 3, 1 });
    if (_appStarten)
    {
      try
      {
        starteApplikationsProzess();
        System.out.println("Zu prfende Applikation gestartet");
        Thread.sleep(getUmgebung().APP_START_VERZOEGERUNG);
      }
      catch (Exception e)
      {
        throw new RuntimeException(e);
      }
    }
    else
    {
      System.out.println("RDS/TMC-Meldungserstellung jetzt extern starten");
      warteAufEingabe();
    }  

    _debug.info("Testzeitpunkt: t = 0");
    setzteIndikatoren(new int[] {2, 2, 2, 3, 2, 3, 3, 3, 3, 3, 3, 4, 5, 1, 1, 3, 3, 1 });

    warte(60000);
    _debug.info("Starte P.1.2\nTestzeitpunkt: t = 60s");
    setzteIndikatoren(new int[] {2, 2, 2, 3, 2, 3, 3, 3, 3, 3, 3, 4, 4, 1, 1, 5, 4, 1 });
    
    warte(60000);
    _debug.info("Testzeitpunkt: t = 120s");
    setzteIndikatoren(new int[] {2, 2, 2, 3, 2, 3, 3, 3, 3, 3, 3, 5, 5, 1, 5, 6, 5, 1 },
        new int[] {100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 65, 69, 100});
    
    warte(60000);
    _debug.info("Testzeitpunkt: t = 180s");
    setzteIndikatoren(new int[] {3, 3, 3, 4, 3, 4, 4, 4, 5, 6, 4, 5, 6, 1, 6, 1, 6, 1 });
        
    warte(20000);
    _debug.info("Testzeitpunkt: t = 200s");
    aendereParameter1();
    
    warte(40000);
    _debug.info("Testzeitpunkt: t = 240s");
    setzteIndikatoren(new int[] {3, 3, 3, 4, 3, 4, 4, 4, 4, 5, 4, 5, 6, 1, 6, 1, 6, 1 });
    
    warte(20000);
    _debug.info("Testzeitpunkt: t = 260s");
    aendereParameter2();
    
    warte(40000);
    _debug.info("Testzeitpunkt: t = 300s");
    setzteIndikatoren(new int[] {3, 3, 3, 4, 3, 4, 4, 4, 4, 5, 4, 5, 6, 1, 6, 1, 6, 1 });
    
    warte(30000);
    _debug.info("Starte P.1.3\nTestzeitpunkt: t = 330s");
    bereinigeStaus();
    aendereParameter3();
    
    warte(10000);
    _debug.info("Testzeitpunkt: t = 340s");
    SystemObject stau = erzeugeStau();
    
    warte(40000);
    _debug.info("Testzeitpunkt: t = 380s");
    //_indikatorVerfahren3[4].send(6);
    //_debug.info("Strfallindikator S3,5  wird mit Zustand \"6\" (" + StoerfallZustand.get(7) + ") belegt");
    //setzteIndikatoren(new int[] {3, 3, 3, 4, 3, 4, 4, 4, 4, 5, 6, 5, 6, 1, 6, 1, 6, 1 });
    setzteIndikatoren(new int[] {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 0, 0, 0 });
    
    warte(20000);
    _debug.info("Starte P.1.4\nTestzeitpunkt: t = 400s");
    setzteIndikatoren(new int[] {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0, 0, 0, 0, 5, 0, 0 });
    aendereParameter4();
    
    warte(20000);
    _debug.info("Testzeitpunkt: t = 420s");
    setzteIndikatoren(new int[] {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 5, 0, 0, 0, 0, 0, 0, 0 });
    
    warte(20000);
    _debug.info("Testzeitpunkt: t = 440s");
    entparametrisiere(stau);
    
    warte(10000);
    _debug.info("Testzeitpunkt: t = 450s");
    aendereParameter5();
    
    warte(170000);
    _debug.info("Testzeitpunkt: t = 620s ca.");
    
    warte(80000);
    _debug.info("Testzeitpunkt: t = 700s");
    parametrisiere(stau);
    
    warte(20000);
    _debug.info("Testzeitpunkt: t = 720s\nEnde der Prfungen Prffall 1" +
        "\nRDS/TMC-Meldungserstellung jetzt beenden");
    warteAufEingabe();
  }
  
  private void aendereParameter1()
  {
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsModell");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
        
    Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(), 
        new DataDescription(atg, asp)).getData();
    if (altParam != null)
    {
      setOriginalParameter(altParam);
      
      Data neuParam = altParam.createModifiableCopy();
      String liste = null;
      String text = "Parameternderungen:";
      
      ReferenceArray refArray = neuParam.getItem("RDSSIVerfahren").asReferenceArray();
      if (refArray.getLength() > 1)
      {
        ReferenceArray array = altParam.getItem("RDSSIVerfahren").asReferenceArray();
        refArray.setLength(refArray.getLength() - 1);
        int j = 0;
        for (int i = 0; i < array.getLength(); i += 1)
        {
          if (i != 1)
          {  
            SystemObject verfahren = array.getReferenceValue(i).getSystemObject();
            refArray.getReferenceValue(j).setSystemObject(verfahren);
            if (liste == null)
            {
              liste = verfahren.getName();
            }
            else
            {
              liste += (", " + verfahren.getName());
            }
            j += 1;
          }  
        }
      }
      text += "\n\tRDSSIVerfahren: " + liste;
        
      neuParam.getItem("RDSDMax").asUnscaledValue().set(1200);
      text += "\n\tRDSDMax: 1200m";
      
      Array dataArray = neuParam.getItem("RDSAusdehnungProLSI").asArray();
      int laenge = dataArray.getLength();
      dataArray.setLength(laenge + 1);
      Data ausdehnung = dataArray.getItem(laenge);
      SystemObject object = getConnection().getDataModel().getObject("mq.bw.mq_3_hfb1_sw"); 
      ausdehnung.getItem("LSI").asReferenceValue().setSystemObject(object);
      ausdehnung.getItem("AusdehnungVor").asUnscaledValue().set(50);
      ausdehnung.getItem("AusdehnungNach").asUnscaledValue().set(450);
      text += "\n\tRDSAusdehnungProLSI fr " + object.getName() + ": -450m, 50m";
      
      asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
      new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam).start();
      _debug.config("Parameter-Aktualisierung gestartet");
      _debug.info(text);
    }
  }
  
  private void aendereParameter2()
  {
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsModell");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
        
    Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(), 
        new DataDescription(atg, asp)).getData();
    if (altParam != null)
    {
      Data neuParam = altParam.createModifiableCopy();
      neuParam.getItem("RDSNichtAddieren").asTextValue().setText("Ja");
      neuParam.getItem("RDSDMaxEinzelStoerfall").asUnscaledValue().set(1000);
      
      asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
      new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam).start();
      _debug.config("Parameter-Aktualisierung gestartet");
      _debug.info("Parameternderungen:" + 
        "\n\tRDSNichtAddieren: Ja" + 
        "\n\tRDSDMaxEinzelStoerfall: 1000m");
    }
  }
  
  private void aendereParameter3()
  {
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsModell");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
        
    Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(), 
        new DataDescription(atg, asp)).getData();
    if (altParam != null)
    {
      Data neuParam = altParam.createModifiableCopy();
      neuParam.getItem("RDSPlausibStauverlaufsanalyse").asTextValue().setText("Ja");
      
      StoerfallZustand von = StoerfallZustand.STAU;
      StoerfallZustand nach = StoerfallZustand.ZAEHFLIESSENDER_VERKEHR;
      
      Array verzMatrix = neuParam.getArray("RDSVerzgerungsmatrix");
      boolean verzGesetzt = false;
      if (verzMatrix != null)
      {  
        for (int i = 0; i < verzMatrix.getLength(); i += 1)
        {
          Data verz = verzMatrix.getItem(i);
          if (verz != null)
          {
            int x = verz.getItem("SituationVon").asUnscaledValue().intValue();
            int y = verz.getItem("SituationNach").asUnscaledValue().intValue();
            if (x == von.id && y == nach.id)
            {
              verz.getItem("Verzgerungswert").asUnscaledValue().set(20);
              verzGesetzt = true;
              break;
            }
          }
        }
      }
      
      asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
      new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam).start();
      _debug.config("Parameter-Aktualisierung gestartet");
      _debug.info("Parameternderungen:" + 
          "\n\tRDSPlausibStauverlaufsanalyse: Ja" + (!verzGesetzt ? "" : 
          "\n\tRDSVerzgerungsmatrix " + von + "->" + nach + ": 40s"));
    }
  }
  
  private void aendereParameter4()
  {
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsModell");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
        
    Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(), 
        new DataDescription(atg, asp)).getData();
    if (altParam != null)
    {
      Data neuParam = altParam.createModifiableCopy();
      neuParam.getItem("RDSBercksichtigeStauverlaufsanalyse").asTextValue().setText("Ja");
      neuParam.getItem("RDSPlausibStauverlaufsanalyse").asTextValue().setText("Nein");
            
      asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
      new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam).start();
      _debug.config("Parameter-Aktualisierung gestartet");
      _debug.info("Parameternderungen:" + 
      "\n\tRDSBercksichtigeStauverlaufsanalyse: Ja" +
      "\n\tRDSPlausibStauverlaufsanalyse: Nein");
    }
  }
  
  private void aendereParameter5()
  {
    AttributeGroup atg = getConnection().getDataModel().getAttributeGroup("atg.rdsModell");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
        
    Data altParam = new ParameterEmpfaenger(getConnection(), getUmgebung().getLms(), 
        new DataDescription(atg, asp)).getData();
    if (altParam != null)
    {
      Data neuParam = altParam.createModifiableCopy();
      neuParam.getItem("RDSPlausibStauverlaufsanalyse").asTextValue().setText("Ja");
            
      asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
      new ParameterSender(getConnection(), getUmgebung().getLms(), new DataDescription(atg, asp), neuParam).start();
      _debug.config("Parameter-Aktualisierung gestartet");
      _debug.info("Parameternderungen:" + 
          "\n\tRDSPlausibStauverlaufsanalyse: Ja");
    }
  }
    
  private SystemObject erzeugeStau()
  {
    DataModel dm = getConnection().getDataModel();
    MutableSet menge = ((ConfigurationObject)getUmgebung().getVmn()).getMutableSet(STAU_MENGE);
    short ext = 1;
    
    SystemObject stau = null;
    
    while(ext < Short.MAX_VALUE && dm.getObject(STAU_PID_PREFIX + ext) != null)
    {
      ext += 1;
    }
    
    DynamicObjectType typ = (DynamicObjectType)dm.getType("typ.stau");
    try
    {
      stau = getDefaultConfigurationArea().createDynamicObject(typ, STAU_PID_PREFIX + ext, STAU_NAME_PREFIX + ext, null);
      menge.add(stau);
    }
    catch (ConfigurationChangeException e)
    {
      _debug.warning("Fehler", e);
      throw new RuntimeException(e);
    }
      
    AttributeGroup atg = dm.getAttributeGroup("atg.situationsEigenschaften");
    Data param = getConnection().createData(atg);
    ReferenceArray array = param.getItem("StraenSegment").asReferenceArray();
    array.setLength(2);
    array.getReferenceValue(0).setSystemObject(dm.getObject("ass.p.11904.p.11905.0"));
    array.getReferenceValue(1).setSystemObject(dm.getObject("ass.p.11905.p.11906.0"));
    param.getItem("StartOffset").asUnscaledValue().set(2900);
    param.getItem("EndOffset").asUnscaledValue().set(2000);
    param.getItem("StartZeit").asTimeValue().setMillis(getConnection().getTime());
    param.getItem("Dauer").asTimeValue().setSeconds(0);
    
    new ParameterSender(getConnection(), stau, new DataDescription(atg, dm.getAspect("asp.parameterVorgabe")), param).start();
    
    _debug.info("Neues dynamisches Stauobjekt " + stau.getName() + " erzeugt:" + 
        "\n\tStart auf ass.p.11904.p.11905.0 bei 2900m\n\tEnde auf ass.p.11905.p.11906.0 bei 2000m");
    
    return stau;
  }
  
  private void entparametrisiere(SystemObject objekt)
  {
    DataModel dm = getConnection().getDataModel();
    ConfigurationObject co = dm.getConfigurationAuthority();
    if (!co.isOfType("typ.parametrierung"))
    {
      throw new UnsupportedOperationException("Objekt " + co.getPid() + " ist nicht vom Typ typ.parametrierung");
    }
    
    AttributeGroup atg = dm.getAttributeGroup("atg.parametrierung");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
        
    Data altParam = new ParameterEmpfaenger(getConnection(), co, new DataDescription(atg, asp)).getData();
    if (altParam != null)
    {
      Data neuParam = altParam.createModifiableCopy();
            
      Array paramSatz = neuParam.getItem("ParameterSatz").asArray();
      paramSatz.setLength(paramSatz.getLength() + 1);
      Data parameter = paramSatz.getItem(paramSatz.getLength() - 1);
      Array spezifikation = parameter.getItem("DatenSpezifikation").asArray();
      spezifikation.setLength(1);
      Data element = spezifikation.getItem(0);
      ReferenceArray objArray = element.getItem("Objekt").asReferenceArray();
      objArray.setLength(1);
      objArray.getReferenceValue(0).setSystemObject(objekt);
      ReferenceArray atgArray = element.getItem("AttributGruppe").asReferenceArray();
      atgArray.setLength(1);
      atgArray.getReferenceValue(0).setSystemObject(dm.getAttributeGroup("atg.situationsEigenschaften"));
      Data einstellungen = parameter.getItem("Einstellungen");
      einstellungen.getItem("Parametrieren").asTextValue().setText("Nein");
      
      asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
      new ParameterSender(getConnection(), co, new DataDescription(atg, asp), neuParam).start();
      _debug.config("Parameter-Aktualisierung gestartet");
      _debug.info("Fr Objekt " + objekt.getName() + " wird \"keine Daten\" empfangen");
    }
    else
    {
      throw new RuntimeException("Parameter konnten nicht gendert werden");
    }
  }
  
  private void parametrisiere(SystemObject objekt)
  {
    DataModel dm = getConnection().getDataModel();
    ConfigurationObject co = dm.getConfigurationAuthority();
    if (!co.isOfType("typ.parametrierung"))
    {
      throw new RuntimeException("Objekt " + co.getPid() + " ist nicht vom Typ typ.parametrierung");
    }
    
    AttributeGroup atg = dm.getAttributeGroup("atg.parametrierung");
    Aspect asp = getConnection().getDataModel().getAspect("asp.parameterSoll");
        
    Data altParam = new ParameterEmpfaenger(getConnection(), co, new DataDescription(atg, asp)).getData();
    if (altParam != null)
    {
      Data neuParam = altParam.createModifiableCopy();
            
      Array paramSatz = neuParam.getItem("ParameterSatz").asArray();
      Data parameter = paramSatz.getItem(paramSatz.getLength() - 1);
      
      Array spezifikation = parameter.getItem("DatenSpezifikation").asArray();
      if (spezifikation.getLength() == 1)
      {
        Data element = spezifikation.getItem(0);
        ReferenceArray objArray = element.getItem("Objekt").asReferenceArray();
        if (objArray.getLength() == 1 &&  objArray.getReferenceValue(0).getSystemObject() == objekt)
        {
          ReferenceArray atgArray = element.getItem("AttributGruppe").asReferenceArray();
          if (atgArray.getLength() == 1 && atgArray.getReferenceValue(0).getSystemObject() == dm.getAttributeGroup("atg.situationsEigenschaften"))
          {
            paramSatz.setLength(paramSatz.getLength() - 1);
            
            asp = getConnection().getDataModel().getAspect("asp.parameterVorgabe");
            new ParameterSender(getConnection(), co, new DataDescription(atg, asp), neuParam).start();
            
            atg = dm.getAttributeGroup("atg.situationsEigenschaften");
            Data param = getConnection().createData(atg);
            ReferenceArray array = param.getItem("StraenSegment").asReferenceArray();
            array.setLength(2);
            array.getReferenceValue(0).setSystemObject(dm.getObject("ass.p.11904.p.11905.0"));
            array.getReferenceValue(1).setSystemObject(dm.getObject("ass.p.11905.p.11906.0"));
            param.getItem("StartOffset").asUnscaledValue().set(2900);
            param.getItem("EndOffset").asUnscaledValue().set(2000);
            param.getItem("StartZeit").asTimeValue().setMillis(getConnection().getTime());
            param.getItem("Dauer").asTimeValue().setSeconds(0);
            
            new ParameterSender(getConnection(), objekt, new DataDescription(atg, dm.getAspect("asp.parameterVorgabe")), param).start();
            
            _debug.info("Fr Objekt " + objekt.getName() + " werden normale Datenstze empfangen");
            return;
          }
        }
      }
    }
    throw new RuntimeException("Parameter konnten nicht gendert werden");
  }
  
  private void bereinigeStaus()
  {
    MutableSet menge = ((ConfigurationObject)getUmgebung().getVmn()).getMutableSet(STAU_MENGE);
    
    List<SystemObject> liste = menge.getElements();
    int n = 0;
    if (liste.isEmpty())
    {
      return;
    }
    for (SystemObject stau: liste)
    {
      if (stau.getPid().startsWith(STAU_PID_PREFIX))
      {
        try
        {
          menge.remove(stau);
          ((DynamicObject)stau).invalidate();
          n += 1;
        }
        catch (ConfigurationChangeException e)
        {
          _debug.error("Fehler", e);
        }
        
      }  
    }
    _debug.config(n + " Stau-Objekt(e) bereinigt");
  }

}

